@@ -1,3 +1,4 @@
+# mypy: disable_error_code="no-untyped-def,no-untyped-call,attr-defined,arg-type,no-any-return,list-item,var-annotated,import,call-overload"
 # Copyright 2016–2021 Julien Danjou
 # Copyright 2016 Joshua Harlow
 # Copyright 2013 Ray Holder
@@ -24,6 +25,7 @@ import warnings
 from contextlib import contextmanager
 from copy import copy
 from fractions import Fraction
+from unittest import mock
 
 import pytest
 
@@ -50,12 +52,19 @@ def _set_delay_since_start(retry_state, delay):
     assert retry_state.seconds_since_start == delay
 
 
-def make_retry_state(previous_attempt_number, delay_since_first_attempt, last_result=None):
+def make_retry_state(
+    previous_attempt_number,
+    delay_since_first_attempt,
+    last_result=None,
+    upcoming_sleep=0,
+):
     """Construct RetryCallState for given attempt number & delay.
 
     Only used in testing and thus is extra careful about timestamp arithmetics.
     """
-    required_parameter_unset = previous_attempt_number is _unset or delay_since_first_attempt is _unset
+    required_parameter_unset = (
+        previous_attempt_number is _unset or delay_since_first_attempt is _unset
+    )
     if required_parameter_unset:
         raise _make_unset_exception(
             "wait/stop",
@@ -69,6 +78,9 @@ def make_retry_state(previous_attempt_number, delay_since_first_attempt, last_re
         retry_state.outcome = last_result
     else:
         retry_state.set_result(None)
+
+    retry_state.upcoming_sleep = upcoming_sleep
+
     _set_delay_since_start(retry_state, delay_since_first_attempt)
     return retry_state
 
@@ -86,9 +98,15 @@ class TestBase(unittest.TestCase):
         rs.idle_for = 1.1111111
         assert repr(rs).endswith("attempt #1; slept for 1.11; last result: none yet>")
         rs = make_retry_state(2, 5)
-        assert repr(rs).endswith("attempt #2; slept for 0.0; last result: returned None>")
-        rs = make_retry_state(0, 0, last_result=tenacity.Future.construct(1, ValueError("aaa"), True))
-        assert repr(rs).endswith("attempt #0; slept for 0.0; last result: failed (ValueError aaa)>")
+        assert repr(rs).endswith(
+            "attempt #2; slept for 0.0; last result: returned None>"
+        )
+        rs = make_retry_state(
+            0, 0, last_result=tenacity.Future.construct(1, ValueError("aaa"), True)
+        )
+        assert repr(rs).endswith(
+            "attempt #0; slept for 0.0; last result: failed (ValueError aaa)>"
+        )
 
 
 class TestStopConditions(unittest.TestCase):
@@ -97,7 +115,9 @@ class TestStopConditions(unittest.TestCase):
         self.assertFalse(r.stop(make_retry_state(3, 6546)))
 
     def test_stop_any(self):
-        stop = tenacity.stop_any(tenacity.stop_after_delay(1), tenacity.stop_after_attempt(4))
+        stop = tenacity.stop_any(
+            tenacity.stop_after_delay(1), tenacity.stop_after_attempt(4)
+        )
 
         def s(*args):
             return stop(make_retry_state(*args))
@@ -110,7 +130,9 @@ class TestStopConditions(unittest.TestCase):
         self.assertTrue(s(4, 1.8))
 
     def test_stop_all(self):
-        stop = tenacity.stop_all(tenacity.stop_after_delay(1), tenacity.stop_after_attempt(4))
+        stop = tenacity.stop_all(
+            tenacity.stop_after_delay(1), tenacity.stop_after_attempt(4)
+        )
 
         def s(*args):
             return stop(make_retry_state(*args))
@@ -162,6 +184,21 @@ class TestStopConditions(unittest.TestCase):
                 self.assertTrue(r.stop(make_retry_state(2, 1)))
                 self.assertTrue(r.stop(make_retry_state(2, 1.001)))
 
+    def test_stop_before_delay(self):
+        for delay in (1, datetime.timedelta(seconds=1)):
+            with self.subTest():
+                r = Retrying(stop=tenacity.stop_before_delay(delay))
+                self.assertFalse(
+                    r.stop(make_retry_state(2, 0.999, upcoming_sleep=0.0001))
+                )
+                self.assertTrue(r.stop(make_retry_state(2, 1, upcoming_sleep=0.001)))
+                self.assertTrue(r.stop(make_retry_state(2, 1, upcoming_sleep=1)))
+
+                # It should act the same as stop_after_delay if upcoming sleep is 0
+                self.assertFalse(r.stop(make_retry_state(2, 0.999, upcoming_sleep=0)))
+                self.assertTrue(r.stop(make_retry_state(2, 1, upcoming_sleep=0)))
+                self.assertTrue(r.stop(make_retry_state(2, 1.001, upcoming_sleep=0)))
+
     def test_legacy_explicit_stop_type(self):
         Retrying(stop="stop_after_attempt")
 
@@ -188,15 +225,23 @@ class TestWaitConditions(unittest.TestCase):
                 self.assertEqual(1, r.wait(make_retry_state(12, 6546)))
 
     def test_incrementing_sleep(self):
-        for start, increment in ((500, 100), (datetime.timedelta(seconds=500), datetime.timedelta(seconds=100))):
+        for start, increment in (
+            (500, 100),
+            (datetime.timedelta(seconds=500), datetime.timedelta(seconds=100)),
+        ):
             with self.subTest():
-                r = Retrying(wait=tenacity.wait_incrementing(start=start, increment=increment))
+                r = Retrying(
+                    wait=tenacity.wait_incrementing(start=start, increment=increment)
+                )
                 self.assertEqual(500, r.wait(make_retry_state(1, 6546)))
                 self.assertEqual(600, r.wait(make_retry_state(2, 6546)))
                 self.assertEqual(700, r.wait(make_retry_state(3, 6546)))
 
     def test_random_sleep(self):
-        for min_, max_ in ((1, 20), (datetime.timedelta(seconds=1), datetime.timedelta(seconds=20))):
+        for min_, max_ in (
+            (1, 20),
+            (datetime.timedelta(seconds=1), datetime.timedelta(seconds=20)),
+        ):
             with self.subTest():
                 r = Retrying(wait=tenacity.wait_random(min=min_, max=max_))
                 times = set()
@@ -283,7 +328,10 @@ class TestWaitConditions(unittest.TestCase):
         self.assertEqual(r.wait(make_retry_state(20, 0)), 1048576)
 
     def test_exponential_with_min_wait_andmax__wait(self):
-        for min_, max_ in ((10, 100), (datetime.timedelta(seconds=10), datetime.timedelta(seconds=100))):
+        for min_, max_ in (
+            (10, 100),
+            (datetime.timedelta(seconds=10), datetime.timedelta(seconds=100)),
+        ):
             with self.subTest():
                 r = Retrying(wait=tenacity.wait_exponential(min=min_, max=max_))
                 self.assertEqual(r.wait(make_retry_state(1, 0)), 10)
@@ -310,7 +358,11 @@ class TestWaitConditions(unittest.TestCase):
         self.assertEqual(r.wait(make_retry_state(10, 100)), 1000)
 
     def test_wait_combine(self):
-        r = Retrying(wait=tenacity.wait_combine(tenacity.wait_random(0, 3), tenacity.wait_fixed(5)))
+        r = Retrying(
+            wait=tenacity.wait_combine(
+                tenacity.wait_random(0, 3), tenacity.wait_fixed(5)
+            )
+        )
         # Test it a few time since it's random
         for i in range(1000):
             w = r.wait(make_retry_state(1, 5))
@@ -326,7 +378,11 @@ class TestWaitConditions(unittest.TestCase):
             self.assertGreaterEqual(w, 5)
 
     def test_wait_triple_sum(self):
-        r = Retrying(wait=tenacity.wait_fixed(1) + tenacity.wait_random(0, 3) + tenacity.wait_fixed(5))
+        r = Retrying(
+            wait=tenacity.wait_fixed(1)
+            + tenacity.wait_random(0, 3)
+            + tenacity.wait_fixed(5)
+        )
         # Test it a few time since it's random
         for i in range(1000):
             w = r.wait(make_retry_state(1, 5))
@@ -478,7 +534,10 @@ class TestWaitConditions(unittest.TestCase):
 
         retrying = Retrying(
             wait=waitfunc,
-            retry=(tenacity.retry_if_exception_type() | tenacity.retry_if_result(lambda result: result == 123)),
+            retry=(
+                tenacity.retry_if_exception_type()
+                | tenacity.retry_if_result(lambda result: result == 123)
+            ),
         )
 
         def returnval():
@@ -562,7 +621,9 @@ class TestRetryConditions(unittest.TestCase):
         self.assertFalse(r(tenacity.Future.construct(1, 1, True)))
 
     def test_retry_and(self):
-        retry = tenacity.retry_if_result(lambda x: x == 1) & tenacity.retry_if_result(lambda x: isinstance(x, int))
+        retry = tenacity.retry_if_result(lambda x: x == 1) & tenacity.retry_if_result(
+            lambda x: isinstance(x, int)
+        )
 
         def r(fut):
             retry_state = make_retry_state(1, 1.0, last_result=fut)
@@ -574,7 +635,9 @@ class TestRetryConditions(unittest.TestCase):
         self.assertFalse(r(tenacity.Future.construct(1, 1, True)))
 
     def test_retry_or(self):
-        retry = tenacity.retry_if_result(lambda x: x == "foo") | tenacity.retry_if_result(lambda x: isinstance(x, int))
+        retry = tenacity.retry_if_result(
+            lambda x: x == "foo"
+        ) | tenacity.retry_if_result(lambda x: isinstance(x, int))
 
         def r(fut):
             retry_state = make_retry_state(1, 1.0, last_result=fut)
@@ -592,7 +655,9 @@ class TestRetryConditions(unittest.TestCase):
 
     def test_retry_try_again(self):
         self._attempts = 0
-        Retrying(stop=tenacity.stop_after_attempt(5), retry=tenacity.retry_never)(self._raise_try_again)
+        Retrying(stop=tenacity.stop_after_attempt(5), retry=tenacity.retry_never)(
+            self._raise_try_again
+        )
         self.assertEqual(3, self._attempts)
 
     def test_retry_try_again_forever(self):
@@ -656,7 +721,7 @@ class NoIOErrorAfterCount:
         """
         if self.counter < self.count:
             self.counter += 1
-            raise IOError("Hi there, I'm an IOError")
+            raise OSError("Hi there, I'm an IOError")
         return True
 
 
@@ -698,7 +763,7 @@ class NoNameErrorCauseAfterCount:
             try:
                 self.go2()
             except NameError as e:
-                raise IOError() from e
+                raise OSError() from e
 
         return True
 
@@ -711,7 +776,7 @@ class NoIOErrorCauseAfterCount:
         self.count = count
 
     def go2(self):
-        raise IOError("Hi there, I'm an IOError")
+        raise OSError("Hi there, I'm an IOError")
 
     def go(self):
         """Raise a NameError with an IOError as cause until after count threshold has been crossed.
@@ -722,7 +787,7 @@ class NoIOErrorCauseAfterCount:
             self.counter += 1
             try:
                 self.go2()
-            except IOError as e:
+            except OSError as e:
                 raise NameError() from e
 
         return True
@@ -763,7 +828,7 @@ class IOErrorUntilCount:
         if self.counter < self.count:
             self.counter += 1
             return True
-        raise IOError("Hi there, I'm an IOError")
+        raise OSError("Hi there, I'm an IOError")
 
 
 class CustomError(Exception):
@@ -850,7 +915,9 @@ def _retryable_test_if_not_exception_type_io(thing):
     return thing.go()
 
 
-@retry(stop=tenacity.stop_after_attempt(3), retry=tenacity.retry_if_exception_type(IOError))
+@retry(
+    stop=tenacity.stop_after_attempt(3), retry=tenacity.retry_if_exception_type(IOError)
+)
 def _retryable_test_with_exception_type_io_attempt_limit(thing):
     return thing.go()
 
@@ -875,28 +942,46 @@ def _retryable_test_with_unless_exception_type_no_input(thing):
 
 @retry(
     stop=tenacity.stop_after_attempt(5),
-    retry=tenacity.retry_if_exception_message(message=NoCustomErrorAfterCount.derived_message),
+    retry=tenacity.retry_if_exception_message(
+        message=NoCustomErrorAfterCount.derived_message
+    ),
 )
 def _retryable_test_if_exception_message_message(thing):
     return thing.go()
 
 
-@retry(retry=tenacity.retry_if_not_exception_message(message=NoCustomErrorAfterCount.derived_message))
+@retry(
+    retry=tenacity.retry_if_not_exception_message(
+        message=NoCustomErrorAfterCount.derived_message
+    )
+)
 def _retryable_test_if_not_exception_message_message(thing):
     return thing.go()
 
 
-@retry(retry=tenacity.retry_if_exception_message(match=NoCustomErrorAfterCount.derived_message[:3] + ".*"))
+@retry(
+    retry=tenacity.retry_if_exception_message(
+        match=NoCustomErrorAfterCount.derived_message[:3] + ".*"
+    )
+)
 def _retryable_test_if_exception_message_match(thing):
     return thing.go()
 
 
-@retry(retry=tenacity.retry_if_not_exception_message(match=NoCustomErrorAfterCount.derived_message[:3] + ".*"))
+@retry(
+    retry=tenacity.retry_if_not_exception_message(
+        match=NoCustomErrorAfterCount.derived_message[:3] + ".*"
+    )
+)
 def _retryable_test_if_not_exception_message_match(thing):
     return thing.go()
 
 
-@retry(retry=tenacity.retry_if_not_exception_message(message=NameErrorUntilCount.derived_message))
+@retry(
+    retry=tenacity.retry_if_not_exception_message(
+        message=NameErrorUntilCount.derived_message
+    )
+)
 def _retryable_test_not_exception_message_delay(thing):
     return thing.go()
 
@@ -946,7 +1031,7 @@ class TestDecoratorWrapper(unittest.TestCase):
         try:
             _retryable_test_with_stop(NoIOErrorAfterCount(5))
             self.fail("Expected IOError")
-        except IOError as re:
+        except OSError as re:
             self.assertTrue(isinstance(re, IOError))
             print(re)
 
@@ -960,7 +1045,9 @@ class TestDecoratorWrapper(unittest.TestCase):
             self.assertTrue(isinstance(n, NameError))
             print(n)
 
-        self.assertTrue(_retryable_test_with_exception_type_custom(NoCustomErrorAfterCount(5)))
+        self.assertTrue(
+            _retryable_test_with_exception_type_custom(NoCustomErrorAfterCount(5))
+        )
 
         try:
             _retryable_test_with_exception_type_custom(NoNameErrorAfterCount(5))
@@ -970,20 +1057,24 @@ class TestDecoratorWrapper(unittest.TestCase):
             print(n)
 
     def test_retry_except_exception_of_type(self):
-        self.assertTrue(_retryable_test_if_not_exception_type_io(NoNameErrorAfterCount(5)))
+        self.assertTrue(
+            _retryable_test_if_not_exception_type_io(NoNameErrorAfterCount(5))
+        )
 
         try:
             _retryable_test_if_not_exception_type_io(NoIOErrorAfterCount(5))
             self.fail("Expected IOError")
-        except IOError as err:
+        except OSError as err:
             self.assertTrue(isinstance(err, IOError))
             print(err)
 
     def test_retry_until_exception_of_type_attempt_number(self):
         try:
-            self.assertTrue(_retryable_test_with_unless_exception_type_name(NameErrorUntilCount(5)))
+            self.assertTrue(
+                _retryable_test_with_unless_exception_type_name(NameErrorUntilCount(5))
+            )
         except NameError as e:
-            s = _retryable_test_with_unless_exception_type_name.retry.statistics
+            s = _retryable_test_with_unless_exception_type_name.statistics
             self.assertTrue(s["attempt_number"] == 6)
             print(e)
         else:
@@ -992,9 +1083,13 @@ class TestDecoratorWrapper(unittest.TestCase):
     def test_retry_until_exception_of_type_no_type(self):
         try:
             # no input should catch all subclasses of Exception
-            self.assertTrue(_retryable_test_with_unless_exception_type_no_input(NameErrorUntilCount(5)))
+            self.assertTrue(
+                _retryable_test_with_unless_exception_type_no_input(
+                    NameErrorUntilCount(5)
+                )
+            )
         except NameError as e:
-            s = _retryable_test_with_unless_exception_type_no_input.retry.statistics
+            s = _retryable_test_with_unless_exception_type_no_input.statistics
             self.assertTrue(s["attempt_number"] == 6)
             print(e)
         else:
@@ -1003,7 +1098,9 @@ class TestDecoratorWrapper(unittest.TestCase):
     def test_retry_until_exception_of_type_wrong_exception(self):
         try:
             # two iterations with IOError, one that returns True
-            _retryable_test_with_unless_exception_type_name_attempt_limit(IOErrorUntilCount(2))
+            _retryable_test_with_unless_exception_type_name_attempt_limit(
+                IOErrorUntilCount(2)
+            )
             self.fail("Expected RetryError")
         except RetryError as e:
             self.assertTrue(isinstance(e, RetryError))
@@ -1011,41 +1108,57 @@ class TestDecoratorWrapper(unittest.TestCase):
 
     def test_retry_if_exception_message(self):
         try:
-            self.assertTrue(_retryable_test_if_exception_message_message(NoCustomErrorAfterCount(3)))
+            self.assertTrue(
+                _retryable_test_if_exception_message_message(NoCustomErrorAfterCount(3))
+            )
         except CustomError:
-            print(_retryable_test_if_exception_message_message.retry.statistics)
+            print(_retryable_test_if_exception_message_message.statistics)
             self.fail("CustomError should've been retried from errormessage")
 
     def test_retry_if_not_exception_message(self):
         try:
-            self.assertTrue(_retryable_test_if_not_exception_message_message(NoCustomErrorAfterCount(2)))
+            self.assertTrue(
+                _retryable_test_if_not_exception_message_message(
+                    NoCustomErrorAfterCount(2)
+                )
+            )
         except CustomError:
-            s = _retryable_test_if_not_exception_message_message.retry.statistics
+            s = _retryable_test_if_not_exception_message_message.statistics
             self.assertTrue(s["attempt_number"] == 1)
 
     def test_retry_if_not_exception_message_delay(self):
         try:
-            self.assertTrue(_retryable_test_not_exception_message_delay(NameErrorUntilCount(3)))
+            self.assertTrue(
+                _retryable_test_not_exception_message_delay(NameErrorUntilCount(3))
+            )
         except NameError:
-            s = _retryable_test_not_exception_message_delay.retry.statistics
+            s = _retryable_test_not_exception_message_delay.statistics
             print(s["attempt_number"])
             self.assertTrue(s["attempt_number"] == 4)
 
     def test_retry_if_exception_message_match(self):
         try:
-            self.assertTrue(_retryable_test_if_exception_message_match(NoCustomErrorAfterCount(3)))
+            self.assertTrue(
+                _retryable_test_if_exception_message_match(NoCustomErrorAfterCount(3))
+            )
         except CustomError:
             self.fail("CustomError should've been retried from errormessage")
 
     def test_retry_if_not_exception_message_match(self):
         try:
-            self.assertTrue(_retryable_test_if_not_exception_message_message(NoCustomErrorAfterCount(2)))
+            self.assertTrue(
+                _retryable_test_if_not_exception_message_message(
+                    NoCustomErrorAfterCount(2)
+                )
+            )
         except CustomError:
-            s = _retryable_test_if_not_exception_message_message.retry.statistics
+            s = _retryable_test_if_not_exception_message_message.statistics
             self.assertTrue(s["attempt_number"] == 1)
 
     def test_retry_if_exception_cause_type(self):
-        self.assertTrue(_retryable_test_with_exception_cause_type(NoNameErrorCauseAfterCount(5)))
+        self.assertTrue(
+            _retryable_test_with_exception_cause_type(NoNameErrorCauseAfterCount(5))
+        )
 
         try:
             _retryable_test_with_exception_cause_type(NoIOErrorCauseAfterCount(5))
@@ -1053,6 +1166,27 @@ class TestDecoratorWrapper(unittest.TestCase):
         except NameError:
             pass
 
+    def test_retry_preserves_argument_defaults(self):
+        def function_with_defaults(a=1):
+            return a
+
+        def function_with_kwdefaults(*, a=1):
+            return a
+
+        retrying = Retrying(
+            wait=tenacity.wait_fixed(0.01), stop=tenacity.stop_after_attempt(3)
+        )
+        wrapped_defaults_function = retrying.wraps(function_with_defaults)
+        wrapped_kwdefaults_function = retrying.wraps(function_with_kwdefaults)
+
+        self.assertEqual(
+            function_with_defaults.__defaults__, wrapped_defaults_function.__defaults__
+        )
+        self.assertEqual(
+            function_with_kwdefaults.__kwdefaults__,
+            wrapped_kwdefaults_function.__kwdefaults__,
+        )
+
     def test_defaults(self):
         self.assertTrue(_retryable_default(NoNameErrorAfterCount(5)))
         self.assertTrue(_retryable_default_f(NoNameErrorAfterCount(5)))
@@ -1070,21 +1204,64 @@ class TestDecoratorWrapper(unittest.TestCase):
             def __call__(self):
                 return "Hello"
 
-        retrying = Retrying(wait=tenacity.wait_fixed(0.01), stop=tenacity.stop_after_attempt(3))
+        retrying = Retrying(
+            wait=tenacity.wait_fixed(0.01), stop=tenacity.stop_after_attempt(3)
+        )
         h = retrying.wraps(Hello())
         self.assertEqual(h(), "Hello")
 
+    def test_retry_function_attributes(self):
+        """Test that the wrapped function attributes are exposed as intended.
+
+        - statistics contains the value for the latest function run
+        - retry object can be modified to change its behaviour (useful to patch in tests)
+        - retry object statistics do not contain valid information
+        """
+
+        self.assertTrue(_retryable_test_with_stop(NoneReturnUntilAfterCount(2)))
+
+        expected_stats = {
+            "attempt_number": 3,
+            "delay_since_first_attempt": mock.ANY,
+            "idle_for": mock.ANY,
+            "start_time": mock.ANY,
+        }
+        self.assertEqual(_retryable_test_with_stop.statistics, expected_stats)
+        self.assertEqual(_retryable_test_with_stop.retry.statistics, {})
+
+        with mock.patch.object(
+            _retryable_test_with_stop.retry, "stop", tenacity.stop_after_attempt(1)
+        ):
+            try:
+                self.assertTrue(_retryable_test_with_stop(NoneReturnUntilAfterCount(2)))
+            except RetryError as exc:
+                expected_stats = {
+                    "attempt_number": 1,
+                    "delay_since_first_attempt": mock.ANY,
+                    "idle_for": mock.ANY,
+                    "start_time": mock.ANY,
+                }
+                self.assertEqual(_retryable_test_with_stop.statistics, expected_stats)
+                self.assertEqual(exc.last_attempt.attempt_number, 1)
+                self.assertEqual(_retryable_test_with_stop.retry.statistics, {})
+            else:
+                self.fail("RetryError should have been raised after 1 attempt")
+
 
 class TestRetryWith:
     def test_redefine_wait(self):
         start = current_time_ms()
-        result = _retryable_test_with_wait.retry_with(wait=tenacity.wait_fixed(0.1))(NoneReturnUntilAfterCount(5))
+        result = _retryable_test_with_wait.retry_with(wait=tenacity.wait_fixed(0.1))(
+            NoneReturnUntilAfterCount(5)
+        )
         t = current_time_ms() - start
         assert t >= 500
         assert result is True
 
     def test_redefine_stop(self):
-        result = _retryable_test_with_stop.retry_with(stop=tenacity.stop_after_attempt(5))(NoneReturnUntilAfterCount(4))
+        result = _retryable_test_with_stop.retry_with(
+            stop=tenacity.stop_after_attempt(5)
+        )(NoneReturnUntilAfterCount(4))
         assert result is True
 
     def test_retry_error_cls_should_be_preserved(self):
@@ -1099,7 +1276,7 @@ class TestRetryWith:
 
     def test_retry_error_callback_should_be_preserved(self):
         def return_text(retry_state):
-            return "Calling %s keeps raising errors after %s attempts" % (
+            return "Calling {} keeps raising errors after {} attempts".format(
                 retry_state.fn.__name__,
                 retry_state.attempt_number,
             )
@@ -1189,7 +1366,10 @@ class TestBeforeAfterAttempts(unittest.TestCase):
         finally:
             logger.removeHandler(handler)
 
-        etalon_re = r"^Retrying .* in 0\.01 seconds as it raised " r"(IO|OS)Error: Hi there, I'm an IOError\.$"
+        etalon_re = (
+            r"^Retrying .* in 0\.01 seconds as it raised "
+            r"(IO|OS)Error: Hi there, I'm an IOError\.$"
+        )
         self.assertEqual(len(handler.records), 2)
         fmt = logging.Formatter().format
         self.assertRegex(fmt(handler.records[0]), etalon_re)
@@ -1206,7 +1386,9 @@ class TestBeforeAfterAttempts(unittest.TestCase):
         handler = CapturingHandler()
         logger.addHandler(handler)
         try:
-            _before_sleep = tenacity.before_sleep_log(logger, logging.INFO, exc_info=True)
+            _before_sleep = tenacity.before_sleep_log(
+                logger, logging.INFO, exc_info=True
+            )
             retrying = Retrying(
                 wait=tenacity.wait_fixed(0.01),
                 stop=tenacity.stop_after_attempt(3),
@@ -1236,7 +1418,9 @@ class TestBeforeAfterAttempts(unittest.TestCase):
         handler = CapturingHandler()
         logger.addHandler(handler)
         try:
-            _before_sleep = tenacity.before_sleep_log(logger, logging.INFO, exc_info=exc_info)
+            _before_sleep = tenacity.before_sleep_log(
+                logger, logging.INFO, exc_info=exc_info
+            )
             _retry = tenacity.retry_if_result(lambda result: result is None)
             retrying = Retrying(
                 wait=tenacity.wait_fixed(0.01),
@@ -1333,21 +1517,21 @@ class TestStatistics(unittest.TestCase):
         def _foobar():
             return 42
 
-        self.assertEqual({}, _foobar.retry.statistics)
+        self.assertEqual({}, _foobar.statistics)
         _foobar()
-        self.assertEqual(1, _foobar.retry.statistics["attempt_number"])
+        self.assertEqual(1, _foobar.statistics["attempt_number"])
 
     def test_stats_failing(self):
         @retry(stop=tenacity.stop_after_attempt(2))
         def _foobar():
             raise ValueError(42)
 
-        self.assertEqual({}, _foobar.retry.statistics)
+        self.assertEqual({}, _foobar.statistics)
         try:
             _foobar()
         except Exception:  # noqa: B902
             pass
-        self.assertEqual(2, _foobar.retry.statistics["attempt_number"])
+        self.assertEqual(2, _foobar.statistics["attempt_number"])
 
 
 class TestRetryErrorCallback(unittest.TestCase):
@@ -1519,7 +1703,9 @@ class TestRetryException(unittest.TestCase):
 
 
 class TestRetryTyping(unittest.TestCase):
-    @pytest.mark.skipif(sys.version_info < (3, 0), reason="typeguard not supported for python 2")
+    @pytest.mark.skipif(
+        sys.version_info < (3, 0), reason="typeguard not supported for python 2"
+    )
     def test_retry_type_annotations(self):
         """The decorator should maintain types of decorated functions."""
         # Just in case this is run with unit-test, return early for py2
